@@ -9,7 +9,7 @@ device = 'cuda:0'
 #  t = svox.N3Tree.load("/home/sxyu/proj/volrend/build/lego_diffscale.npz",
 #  t = svox.N3Tree.load("quant_exp_16.npz",
 #  t = svox.N3Tree.load("/home/sxyu/proj/volrend/build/nerf_synthetic_v2/lego_v2.npz",
-t = svox.N3Tree.load("/home/sxyu/proj/volrend/build/mic_sg_sp.npz",
+t = svox.N3Tree.load("/f/fastnerf/oct/misc/mic_sg_sp.npz",
                       map_location=device)
 #  t = svox.N3Tree(map_location=device)
 r = svox.VolumeRenderer(t)
