@@ -5,8 +5,10 @@ import matplotlib.pyplot as plt
 
 device = 'cuda:0'
 
-t = svox.N3Tree(map_location=device)
-t[0, 0, 0, :3] = 0.0
+t = svox.N3Tree(map_location=device,
+         #  data_dim=1 * 3 + 1, data_format="SH1")
+        data_dim=4 * 3 + 1, data_format="SH4")
+t[0, 0, 0, :-1] = 0.0
 t[0, 0, 0, -1:] = 0.5
 r = svox.VolumeRenderer(t)
 print(t)
@@ -24,17 +26,24 @@ ray_dir = torch.tensor([[0.0, 0.0, 1.0]], device=device)
 ray = svox.Rays(origins=ray_ori, dirs=ray_dir, viewdirs=ray_dir)
 print('GRADIENT DESC')
 for i in range(100):
+    t[:, 1:4] = 0
+    t[:, 5:8] = 0
+    t[:, 9:12] = 0
     rend = r(ray, cuda=True)
     if i % 2 == 0:
         print(rend.detach()[0].cpu().numpy())
     #  rend.retain_grad()
-    torch.abs(rend - target).sum().backward()
+    ((rend - target) ** 2).sum().backward()
     #  print('rend grad', rend.grad)
     #  print('data grad', t.data.grad)
     #  print('data', t.data)
     #  print(t.data.grad)
 
-    t.data.data -= t.data.grad
+    t.data.data -= 2.5 * t.data.grad
+    t[:, 1:4] = 0
+    t[:, 5:8] = 0
+    t[:, 9:12] = 0
+    #  print(t.data)
     t.zero_grad()
 
 print('TARGET')
